/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/
#ifndef TEXTEDIT_H
#define TEXTEDIT_H
//-----------------------------------------------------------------------------
#include <QTextEdit>
#if defined(_MSC_VER)
#include <mgl2/define.h>
#endif
//-----------------------------------------------------------------------------
class QCompleter;
class Numb;
//-----------------------------------------------------------------------------
class TextEdit : public QTextEdit
{
	Q_OBJECT
public:
	TextEdit(QWidget *parent = 0);
	void setCompleter(QCompleter *c);
	QCompleter *completer() const {	return c;	}
	int getFirstVisibleBlockId();
	void lineNumberAreaPaintEvent(QPaintEvent *event);
	int lineNumberAreaWidth();

public slots:
	void resizeEvent(QResizeEvent *e);
	void setErrMessage(const QString &s);

protected:
	void keyPressEvent(QKeyEvent *e);
	void focusInEvent(QFocusEvent *e);
	bool isErrLine(int line) const;

private slots:
	void insertCompletion(const QString &completion);
	void highlight();
	void updateLineNumberAreaWidth(int newBlockCount);
	void updateLineNumberArea(QRectF /*rect_f*/);
	void updateLineNumberArea(int /*slider_pos*/);
	void updateLineNumberArea();

private:
	QString textUnderCursor() const;
	QCompleter *c;
	QWidget *lineNumberArea;
	std::vector<int> err;
};
//-----------------------------------------------------------------------------
class LineNumberArea : public QWidget
{
	Q_OBJECT
public:
	LineNumberArea(TextEdit *editor);
	QSize sizeHint() const;
protected:
	void paintEvent(QPaintEvent *event);
private:
	TextEdit *codeEditor;
};

//-----------------------------------------------------------------------------
#endif // TEXTEDIT_H
//-----------------------------------------------------------------------------
